<?php
/*
Plugin Name: OneScreen Video
Plugin URI: http://onescreen.com
Description: Utilize the [onescreen] shortcode to embed OneScreen's video player. *UPDATE: Now requires user generated token from www.mediagraph.com account to work
Version: 1.3.2
Author: Steven Ton [Edited by Michael Trieu]
Author URI: http://onescreen.com
*/

class OS_Shortcode {
    public $options;

    public function __construct() {
        // get plugin options - account id & passthrough playlist id
        $this->options = get_option('os_plugin_options');

        // add a filter so that shortcodes are executed in widget areas
        add_filter('widget_text', 'do_shortcode');

        // add our shortcode so wordpress recognizes "[onescreen]"
        add_shortcode('onescreen', array($this, 'shortcode'));

        // remove our shortcode from RSS2 feeds
        add_action('do_feed_rss2', array($this, 'removeShortcodeFomRss'), 2);

        add_action('admin_menu', array($this, 'add_menu_page'));
        add_action('admin_init', array($this, 'register_settings_and_fields'));

        // GET REQUEST URLS
        define('CONTENTS_URL', 'http://api2.onescreen.net/v2/contents.json');
        define('SHARED_CONTENTS_URL', 'http://api2.onescreen.net/v2/contents/shared.json');
        define('OS_TOKEN', $this->options['os_mediagraph_token_setting']);
    }

    public function removeShortcodeFomRss($shortcode) {
        remove_shortcode('onescreen');
    }

    private function target_div_counter() { 
        STATIC $count = 0;
        $count++;
        return $count;
    }

    public function shortcode($attributes) {

        $scripts_to_include = array('os_widgets' => 'http://cdn.onescreen.net/os/static/apps/s/_onescreen.js');

        // default values for all properties
        // shortcode_atts is used to set default values for our attributes
        $attributes = shortcode_atts(
            array(
                'item' => '',
                'widget_id' => $this->options['os_default_widget_id_setting'],  
                'target_div' => 'os_dmp_embed_' . $this->target_div_counter(),
                'custom_render_css' => '',
                'custom_render_script' => '',
                'helper_js' => '',
                'auto_play' => '',
                'width' => '',
                'height' => '',
                'ads' => '',
                'skin' => '',
                'start_index' => '',
                'dock' => '',
                'custom_render' => '',
                'companion_target' => '',
                'force_companions' => '',
                'player_fill' => '',
                'force_html5' => '',
                'playlist_id' => '',
                'app_id' => ''
            ), $attributes
        );

        // pulls out all the keys and uses those as variables for the values
        extract( $attributes );

        if (empty($widget_id)) return;

        // if attempting to use passthrough playlist or override "media" property
        if (!empty($item) || !empty($playlist_id)){
            // VERIFICATION
            $account = $this->os_authenticate_token();
            if ($account == false || empty($this->options['os_mediagraph_token_setting']) ) return '<h4><span style="color:red;">ERROR:</span> Please Enter Account Token under OneScreen Account Settings</h4>';
            // Get first item in array to set up info
            $video = $account[0];
            $video_id = $video->id;
            $passthrough_uri = $video->passthrough_uri;
            if (!empty($item)){
                $passthrough_uri = str_replace($video_id, '', $passthrough_uri);
            }
            else {
                if (strripos($passthrough_uri, "/") !== false) $passthrough_uri = substr($passthrough_uri, 0, strripos($passthrough_uri, "/")) . '/' . $playlist_id . '.xml?plf=os' ;
            }
        }

        if (!empty($helper_js)) {
            $scripts_to_include['widget_js'] = $helper_js;
        }

        foreach ($scripts_to_include as $handler => $script_src) {
            wp_register_script($handler, $script_src);
            wp_enqueue_script($handler, $script_src);
        }
        $handlers = array_keys($scripts_to_include);

        $config = array();

        if (!empty($custom_render_css)) {
            $config['customRenderCSS'] = $custom_render_css;
        }

        if (!empty($custom_render_script)) {
            $config['customRenderScript'] = $custom_render_script;
        }

        if (!empty($companion_target)) {
            $config['force_companions'] = 'true';
            $config['companionTarget'] = array(array(
                'width' => '300',
                'height' => '250',
                'id' => $companion_target
            ));
        }

        if (!empty($item) || !empty($playlist_id)) {
            if (!empty($item)){
                $item = str_replace(' ', '', $item);
                $config['media'] = $passthrough_uri . $item;
            }
            else $config['media'] = $passthrough_uri;
        }

        // backwards compatibility for widget_id => app_id
        // if app_id exists, then overwrite current widget_id value with app_id value
        if (!empty($app_id)){$attributes['widget_id'] = $app_id; $widget_id = $app_id;}

        // attributes that get passed but are not a part of valid overrides
        if (!empty($attributes)) {
            $non_overrides_attributes = array_flip(array('item', 'widget_id', 'app_id', 'target_div', 'custom_render_css', 'custom_render_script', 'helper_js', 'companion_target'));
            $overrides_attributes = array_diff_key($attributes, $non_overrides_attributes);

            if (!empty($overrides_attributes)) {
                // loop through and set all overrides
                foreach($overrides_attributes as $key=>$value) {
                    if (!empty($value)) {
                        $key_exploded = explode('_', $key);
                        if (count($key_exploded) > 1) {
                            foreach($key_exploded as $index => $key_snippet) {
                                if ($index == 0) {
                                    $key = $key_exploded[$index];
                                } else {
                                    $key .= ucfirst($key_exploded[$index]);
                                }
                            }
                        }
                        $config[$key] = $value;
                    }
                }
            }
        }

        // data attribute media
        $passthrough_url = 'data-media="' . $config['media'] . '"';

        $target_div = "<div id='$target_div' class='load-os-app' ".attributes_to_data($attributes, $config, $passthrough_url)."></div>";

        return $target_div;
        $count++;
    }

    public function add_menu_page() {
        add_options_page('OneScreen Account', 'OneScreen Account', 'administrator', __FILE__, array($this, 'display_options_page'));
    }

    public function display_options_page() {
        ?>
        <div class="wrap">
        <h2>OneScreen Account Settings</h2>

        <form method="post" action="options.php" enctype="multipart/form-data">
            <?php settings_fields('os_plugin_options'); ?>
            <?php do_settings_sections(__FILE__); ?>

            <p class="submit">
                <input name="submit" type="submit" class="button-primary" value="Save Changes"/>
        </form>
        <?php
    }

    public function register_settings_and_fields() {
        register_setting('os_plugin_options', 'os_plugin_options');
        add_settings_section('os_settings_section', 'Account Settings', array($this, 'os_settings_section_cb'), __FILE__);
        add_settings_field('os_mediagraph_token', 'Enter Account Token:', array($this, 'os_mediagraph_token_setting'), __FILE__, 'os_settings_section');
        add_settings_field('os_default_widget_id', 'Default Widget ID: ', array($this, 'os_default_widget_id_setting'), __FILE__, 'os_settings_section');
    }

    public function os_settings_section_cb() {
        // optional
    }

    public function os_mediagraph_token_setting(){
        $this->options['os_mediagraph_token_setting'] = empty($this->options['os_mediagraph_token_setting']) ? '' : $this->options['os_mediagraph_token_setting'];
        echo "<input name='os_plugin_options[os_mediagraph_token_setting]' type='text' value='{$this->options['os_mediagraph_token_setting']}' style='width:50%;' />";
    }

    public function os_default_widget_id_setting() {
        $this->options['os_default_widget_id_setting'] = empty($this->options['os_default_widget_id_setting']) ? '' : $this->options['os_default_widget_id_setting'];
        echo "<input name='os_plugin_options[os_default_widget_id_setting]' type='text' value='{$this->options['os_default_widget_id_setting']}' style='width:50%;' />";
    }

    public function os_authenticate_token($token = OS_TOKEN){
        // uses curl to do GET request on url
        if (!function_exists('verify_url')){
            function verify_url($url){
                $response = wp_remote_get($url);
                $results = $response['body'];
                $results = json_decode($results);
                return $results;
            }
        }

        $url = CONTENTS_URL . '?token=' . $token . '&limit=1';
        $results = verify_url($url);

        // if account doesn't own any videos, check shared content
        if (empty($results)) {
            $url = SHARED_CONTENTS_URL . '?token=' . $token . '&limit=1';
            $results = verify_url($url);
        }

        // check if token gets rejected
        if ($results->error_code == 401 || stripos($results->error_message, 'token rejected') ) $results = false;

        return $results;
    }

}

// adds the 'data-' attributes to the target div so that script can grab it for the .load calls
function attributes_to_data($attributes, $config, $passthrough_url){
    $data_string = $passthrough_url . ' ';
    foreach ($attributes as $attribute_key => $attribute_value){
        // remove key if empty value
        if (empty($attribute_value)) {
            unset($attributes[$attribute_key]);
            continue;
        }
        if (stripos($attribute_key, 'widget_id') !== false) $data_string .= 'data-appid' . '="' . $attribute_value . '" ';
        else $data_string .= 'data-' . $attribute_key .'="' . $attribute_value . '" ';
    }
    return $data_string;
}

new OS_Shortcode();
?>
